
/*
==============================================================================
Name:		RGB_light_3.c
Author:		CAPEK Lukas
Created:	5/2008
Chip:		Atmega8
Frequency: 	CPU 4000000
Hardware:	infra senzor to INT0
			Timer 0 for count lenght of pulses infra
			Timers 1 and 2 for PWM 
============================================================================
*/

#define F_CPU 4000000


#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/io.h>
#include <util/delay_basic.h>
#include <util/delay.h>
#include "RGB_light.h"



static uint16_t  speed_array[] PROGMEM={0,speed_1,speed_2,speed_3,speed_4,speed_5,speed_6,speed_7,speed_8,speed_9,speed_10};
static uint16_t  hue_array[] PROGMEM={0,red,yellow,green,light_blue,blue,violet};

volatile uint8_t value,saturation;  // value is brightness<0,31>
volatile uint8_t timer_high; // timer_high create with TCNT0 16bit counter to count lenght for pulses
volatile uint8_t code_Low_byte; // store last byte of infra code
volatile uint8_t standby;     // stanby mode, 0xFF means active
volatile uint8_t colour_aut;	// if is 0xFF automatic change of colour is enable
volatile uint8_t compare;      // if is 0xFF you can compare receive code
volatile uint8_t counter_st_pulses; // count how long is button press
volatile uint8_t blinking_led;
volatile uint8_t auto_power_off;
volatile uint8_t timer_auto_power_off;
volatile uint8_t time_power_off;
volatile uint8_t direct;
volatile uint16_t hue,my_timer,fast; 
volatile uint32_t code;   // receive code 


int main (void)
{
	// non-inverting, 3 * FAST PWM mode, 8 bit mode, divider 8
	TCCR1A|=(1<<COM1A1)|(0<<COM1A0)|(1<<COM1B1)|(0<<COM1B0)|(1<<WGM10);
	TCCR1B|=(1<<WGM12)|(1<<CS11);
	TCCR2 |=(1<<WGM20)|(1<<WGM21)|(1<<COM21)|(0<<COM20)|(1<<CS21);

	fast = ADCH;

	TCCR0|=(1<<CS01);  	// timer 0 set,divider 8
	MCUCR|=(1<<ISC01)|(0<<ISC00); // falling edge 
	GIMSK|=(1<<INT0); // external interrupt 0 request enable
	TIMSK = (1<<TOIE0); 	// timer 0 overflow interrupt
		
	sei();  
	value = 31;         // max brightness is 31
	saturation = 0xFF;
	standby = enable;		// sleep mode
	fast = speed_1;  // fast change of hue


while(1)
	{

	// decode power 
	if(compare==enable)       
		{
		if(code_Low_byte==power)  
			{
			standby=~standby;
			if(standby==0) wake_up(); else go_sleep();
			code_Low_byte=0;
			code=0;
			}
		}
	
	// if is stanby disable we can compare other codes
	if(compare==enable && standby==disable)
		{
		switch(code_Low_byte)
		{
		case ch_up:	hue+=step_hue;colour_aut=0;if(hue>359 && hue<400)hue=0;break;
		case ch_down: 	hue-=step_hue;colour_aut=0;if(hue>400)hue=359;break;	
		case Vol_down:	if(value>8) value--;break;
		case Vol_up:	if(value<31) value++;break;
		case g_yellow:	if(saturation>120) saturation-=step_saturation; break;
		case b_blue:	if(saturation<254)saturation+=step_saturation; break;
		case g_green:		colour_aut=enable;direct=~direct;code_Low_byte=0;code=0;break;
		case r_red:		colour_aut=disable;code_Low_byte=0;code=0;break;
		
		case one:		push_button(1); break;			
		case two:		push_button(2); break;
		case three:		push_button(3); break;
		case four:		push_button(4); break;
		case five:		push_button(5); break;
		case six:		push_button(6); break;
		case seven:		push_button(7); break;
		case eight:		push_button(8); break;
		case nine:		push_button(9); break;
		case zero_ten:		push_button(10); break;

		case mute:		blinking_led=enable;
						auto_power_off=setting_mode;
						code_Low_byte=0;code=0;break;
		default: break;
		}
		compare=disable;
		HSVtoRGB(hue,saturation,linear_value(value));
		}


	};
}


//-------------evaluate buttons 1-10-----------------
void push_button(uint8_t button)
{
	if(colour_aut==enable && auto_power_off!=setting_mode)
		fast=pgm_read_word(&speed_array[button]);
	else
		{
		if(auto_power_off==setting_mode) 
			{
			timer_auto_power_off=0;
			time_power_off=button*10;
			auto_power_off=enable;
			blinking_led=disable;
			}
		else 
			{
			if(button<7) hue=pgm_read_word(&hue_array[button]);
			}
		}				
	code_Low_byte=0;
	code=0;
}

//  ------wake up mode---------
void wake_up(void)
{
	uint8_t temp=0;
	DDRB = 0xFF;
	while(temp++!=linear_value(value))
		{
		HSVtoRGB(hue,saturation,temp);
		_delay_ms(5);
		}
}

// --------sleep mode-----------
void go_sleep(void)
{
	uint8_t temp;
	standby=enable;
	auto_power_off=disable;
	colour_aut=disable;

	temp=linear_value(value);
	while(temp--)
		{
		HSVtoRGB(hue,saturation,temp);
		_delay_ms(5);
		}
	DDRB = 0x00;
}

// -------- conversion to linear value --------------
uint8_t linear_value(uint8_t x)
{	
	if(x>31) return 255;
	switch(x/8)
	{
	case 0:	 x=x*2+2;break;  // D(f)=<0,7>,H(f)=<2,16>
	case 1:	 x=x*6-26;break; // D(f)=<8,15>,H(f)=<22,64>
	case 2:	 x=x*8-56;break; // D(f)=<16,23>,H(f)=<72,128>	
	case 3:	 x=x*16-241;break; // D(f)=<24,31>,H(f)=<143,255>
	default: break;
	}
	return x;
}



// ------------------HSV to RGB conversion-----------------
void HSVtoRGB(unsigned int h, unsigned char s, unsigned char v )
{
	long q,t,p,f;
	unsigned char i;

	if(s==0) 
		{
		r = g = b = v;  // achromatic (grey)
		return;
		}

	 i =  h / 60;
     f =  h % 60;	// factorial part of h
	 p =  ((v * (255 - s)) / 255) ;
	 q =  ((v * (15049 - s * f)) / 15049);
	 t =  ((v * (15049 - s * (59 - f))) / 15049);

	switch(i)   
	   	{
		case 0: r = v ;g = t ;b = p ;break;
		case 1: r = q ;g = v ;b = p ;break;
		case 2: r = p ;g = v ;b = t ;break;
		case 3: r = p ;g = q ;b = v ;break;
		case 4:	r = t ;g = p ;b = v ;break;
		default:r = v ;g = p ;b = q ;break; 
		}
} 

// ---------count lenght of pulses----------------
ISR(TIMER0_OVF_vect)
{
	my_timer++;

	if(my_timer==0) 
		timer_auto_power_off++;

	if((timer_auto_power_off==time_power_off) && (auto_power_off==enable))
	  go_sleep();

	if(++timer_high==255) 
		{
		counter_st_pulses=0;
		code=0;
		timer_high=254;
		}

// automatic change colour, "fast" is fast of change hue
	if((colour_aut==enable) && (my_timer % fast==0)) 
		{
		if(direct==0) 
			{
			hue++;if(hue>359 && hue<400) hue=0;
			}
		else 
			{
			hue--;if(hue>400) hue=359;
			}
		HSVtoRGB(hue,saturation,linear_value(value));
		}

 	if(standby==disable)
		{
		if((blinking_led==enable) && ((my_timer % 2000)<1000))
	   	 DDRB = 0x00; else DDRB = 0xFF;
		}

}

// -----------interrupt for decode infra code----------------
ISR(INT0_vect)
{	
	uint16_t pulse_time;
	pulse_time  = ((unsigned int)(timer_high)<<8) | (unsigned int)(TCNT0);
	
	timer_high=0;
	TCNT0=0;

	 //start bit sequence or holding key
	if((pulse_time>start_down) && (pulse_time<start_up)) 
		{
		compare=enable;
		if(counter_st_pulses!=255) counter_st_pulses++;
		if(counter_st_pulses==2)		// after second puls in code are validyty data
			{
			code_Low_byte=(unsigned char)(code);
			}
		}

	//for write log 1 to buffer
	if((pulse_time>log_1_down) && (pulse_time<log_1_up)) 
		{
		code = (code<<1);
		code|= 0b00000001;
		}

	//for write log 0 to buffer
	if((pulse_time>log_0_down) && (pulse_time<log_0_up)) 
		{
		code = (code<<1);
		}	
}
